/* gtktest.c -- a silly demonstration program for gtk.

Compile this with 
cc -Wall -g `gtk-config --cflags` `gtk-config --libs` gtktest.c -o gtktest
(under a reaonable shell) or use gmake with a Makefile like

GLIBFLAGS = `gtk-config --cflags` `gtk-config --libs` 
CFLAGS += -Wall -g $(GLIBFLAGS)

%.o:%.c
	$(CC) $(CFLAGS) -c $<

gtktest: gtktest.c

or somesuch (you could get fancier than this...)

If you insist, you may use this code for whetever purpose you
can imagine (I'd be hard pressed to find one, though).

Markus Demleitner (msdemlei@cl.uni-heidelberg.de) 2002
*/

#include <gtk/gtk.h>
#include <stdlib.h>


/* Example callback */
static void doLs(GtkWidget *w, gpointer data)
{
	if (data) system("ls -l");
	else system("ls");
}

/* Here we use the system function to execute a shell command */
static void doTop(GtkWidget *w, gpointer data)
{
	system("xterm -e top&");
}

/* another callback (this gets boring) */
static void doMem(GtkWidget *w, gpointer data)
{
	system("free");
}


GtkWidget *makeMenuBar(GtkWindow *window)
/* returns a menu bar, ready made with all the callbacks, created
   through an ItemFactory.  The keyboard bindings (accelerators)
	 are attached to window */
{
	static GtkItemFactoryEntry menu_items[] = {
		{ "/_File", NULL, NULL, 0, "<Branch>"},
		{ "/File/_Directory", "<control>d", doLs, 0, NULL},
		{ "/File/_Long directory", "<control>l", doLs, 1, NULL},
		{ "/File/_Quit", "<control>q", gtk_main_quit, 0, NULL},
		{ "/_Show", NULL, NULL, 0, "<Branch>"},
		{ "/Show/_Processes", "<control>p", doTop, 0, NULL},
		{ "/Show/_Memory", "<control>m", doMem, 0, NULL},
	};
	GtkAccelGroup *accel_group = gtk_accel_group_new();
	GtkItemFactory *item_factory = gtk_item_factory_new(
		GTK_TYPE_MENU_BAR, "<main>", accel_group);
	
	gtk_item_factory_create_items(item_factory, sizeof(menu_items)/
		sizeof(GtkItemFactoryEntry), menu_items, NULL);
	gtk_window_add_accel_group(window, accel_group);
	return gtk_item_factory_get_widget(item_factory, "<main>");
}


GtkWindow *openMainWindow(char *title, int xSz, int ySz,
	GtkBox **mainBox)
/* returns a window with a box in it.  The purpose of the box
   is to allow more than one widget to get packed in it.  The
	 mainBox is returned through a double pointer. */
{
	GtkWindow *window=GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));

  /* The following makes sure that the window's close button works */
	gtk_signal_connect(GTK_OBJECT(window), "destroy",
		GTK_SIGNAL_FUNC(gtk_main_quit), "WM destroy");
	gtk_window_set_title(window, title);
	/* We want to be at least this big -- this is actually a method
	   every widget supports, so we cast window accordingly. */
	gtk_widget_set_usize(GTK_WIDGET(window), xSz, ySz);

	*mainBox = GTK_BOX(gtk_vbox_new (FALSE, 1));
	gtk_container_border_width(GTK_CONTAINER(*mainBox), 1);
	gtk_container_add(GTK_CONTAINER(window), GTK_WIDGET(*mainBox));
	gtk_widget_show(GTK_WIDGET(window));
	gtk_widget_show(GTK_WIDGET(*mainBox));

	return window;
}


GtkMenuBar *addMenuBar(GtkWindow *window, GtkBox *mainBox)
/* adds a menu bar to mainBox and the corresponding keyboard
   bindings to window -- mainly a wrapper for makeMenuBar */
{
	GtkWidget *menuBar = makeMenuBar(window);
	gtk_box_pack_start(GTK_BOX(mainBox), menuBar, FALSE, TRUE, 0);
	gtk_widget_show(menuBar);
	return GTK_MENU_BAR(menuBar);
}


void fillDirBox(GtkBox *targetBox)
/* packs buttons with directory-related commands into targetBox */
{
	GtkWidget *button;

	button = gtk_button_new_with_label("Directory");
	gtk_signal_connect(GTK_OBJECT(button), "pressed",
		GTK_SIGNAL_FUNC(doLs), 0);
	gtk_box_pack_start(targetBox, button, 1, 1, 0);
	gtk_widget_show(button);

	/* Don't do this at home.  Usually, you want to retain
	a reference to a button, e.g., to free it when no longer
	needed */
	button = gtk_button_new_with_label("Long Directory");
	gtk_box_pack_start(targetBox, button, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(button), "pressed",
		GTK_SIGNAL_FUNC(doLs), (void*)1);
	gtk_widget_show(button);
}


void fillActionBox(GtkBox *targetBox)
/* packs buttons with memory and process infos into targetBox */
{
	GtkWidget *button;

	button = gtk_button_new_with_label("Processes");
	gtk_box_pack_start(targetBox, button, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(button), "pressed",
		GTK_SIGNAL_FUNC(doTop), 0);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Memory");
	gtk_box_pack_start(targetBox, button, 1, 1, 0);
	gtk_signal_connect(GTK_OBJECT(button), "pressed",
		GTK_SIGNAL_FUNC(doMem), 0);
	gtk_widget_show(button);
}


static void changeText(GtkWidget *w, gpointer data)
/* changes the text in a label contained in the GtkBin w (usually
   this should be a button) to the text pointed to by data */
{
	GtkLabel *label = GTK_LABEL(GTK_BIN(w)->child);
	gtk_label_set_text(label, (gchar*)data);
}


GtkWidget *makeFancyQuitButton(void)
/* returns a quit-button that changes its text to Yelp when the
   mouse pointer hovers over it */
{
	GtkWidget *button = gtk_button_new_with_label("Quit");
	gtk_signal_connect(GTK_OBJECT(button), "pressed",
		GTK_SIGNAL_FUNC(gtk_main_quit), 0);
	gtk_signal_connect(GTK_OBJECT(button), "enter",
		GTK_SIGNAL_FUNC(changeText), "Yelp");
	gtk_signal_connect(GTK_OBJECT(button), "leave",
		GTK_SIGNAL_FUNC(changeText), "Quit");
	return button;
}


void addButtonBoxes(GtkBox *mainBox)
/* packs the button panels into mainBox */
{
	GtkBox *upperBox = GTK_BOX(gtk_hbox_new(0, 0));
	GtkBox *lowerBox = GTK_BOX(gtk_hbox_new(0, 0));
	GtkWidget *button = makeFancyQuitButton();

	fillDirBox(upperBox);
	fillActionBox(lowerBox);
	gtk_box_pack_start(mainBox, GTK_WIDGET(upperBox), 1, 1, 0);
	gtk_widget_show(GTK_WIDGET(upperBox));
	gtk_box_pack_start(mainBox, GTK_WIDGET(lowerBox), 1, 1, 0);
	gtk_widget_show(GTK_WIDGET(lowerBox));

	gtk_box_pack_start(mainBox, button, 0, 0, 5);
	gtk_widget_show(button);
}


int main(int argc, char *argv[])
{
	GtkWindow *window;
	GtkBox *mainBox;

	gtk_init(&argc, &argv);
	
	window = openMainWindow("GTK test", 200, 120, &mainBox);
	addMenuBar(window, mainBox);
	addButtonBoxes(mainBox);

	gtk_main();

	return 0;
}

// vim: ts=2
